// Copyright (C) 2017 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:significant reason:default

#ifndef QTSERIALBUSGLOBAL_H
#define QTSERIALBUSGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtSerialBus/qtserialbus-config.h>
#include <QtSerialBus/qtserialbusexports.h>

#endif // QTSERIALBUSGLOBAL_H
