#ifndef QT_FEATURES_SerialBus_src_serialbus_qtserialbus_config_p_h_H
#define QT_FEATURES_SerialBus_src_serialbus_qtserialbus_config_p_h_H

#define QT_FEATURE_socketcan 1

#define QT_FEATURE_socketcan_fd 1


#endif // QT_FEATURES_SerialBus_src_serialbus_qtserialbus_config_p_h_H
